#include "buffer.h"

// Inline extern prototypes

bool LSUP_buffer_eq(const LSUP_Buffer *buf1, const LSUP_Buffer *buf2);
int LSUP_buffer_cmp(const LSUP_Buffer *buf1, const LSUP_Buffer *buf2);


LSUP_rc
LSUP_buffer_new(size_t size, LSUP_Buffer *buf)
{
    LSUP_Buffer *tmp;
    CRITICAL(tmp = malloc (sizeof (*tmp)));
    buf->addr = NULL; // TODO redundant?

    RCCK (LSUP_buffer_reset(buf, size));

    *buf = *tmp;

    return LSUP_OK;
}


LSUP_rc
LSUP_buffer_reset(LSUP_Buffer *buf, size_t size)
{
    //TRACE("Buffer Size: %lu\n", size);
    CRITICAL (buf->addr = realloc (buf->addr, size));
    buf->size = size;

    return LSUP_OK;
}

/**
 * Print a byte string of a given length in a human-readable format.
 *
 * The string is printed in Python style: printable characters are output
 * literally, and non-printable ones as hex sequences.
 */
void LSUP_buffer_print(const LSUP_Buffer *buf) {
    for (size_t i = 0; i < buf->size; i++) {
        char chr = ((char*)buf->addr)[i];
        if (isprint(chr)) {
            fputc(chr, stdout);
        } else {
            printf("\\x%02x", chr);
        }
    }
    printf("\n");
}


LSUP_rc
LSUP_buffer_copy(const LSUP_Buffer *src, LSUP_Buffer *dest)
{
    LSUP_Buffer *tmp;
    RCCK (LSUP_buffer_reset(tmp, src->size));

    memcpy(tmp->addr, src->addr, src->size);

    *dest = *tmp;

    return LSUP_OK;
}


void LSUP_buffer_free(LSUP_Buffer *buf){
    if (LIKELY (buf)) free(buf->addr);
    free(buf);
}

